local _, iTF = ...
--iTF.barTex = 'Interface\\TARGETINGFRAME\\UI-StatusBar'
iTF.barTex = 'Interface\\Buttons\\WHITE8x8'
iTF.font = 'Fonts\\ARIALN.TTF'
iTF.fontSize = 11

iTF.backdrop = {
	bgFile = 'Interface\\Buttons\\WHITE8x8',
	edgeFile = 'Interface\\Buttons\\WHITE8x8',
	edgeSize = 1,
	insets = {
		left = -1,
		right = -1,
		top = -1,
		bottom = -1,
	}
}
local addon = CreateFrame('frame')
addon:RegisterEvent('ADDON_LOADED')
addon:SetScript('OnEvent', function(self, event, ...)
	self[event](self, ...)
end)
local function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end
function iTF:print(msg)
	if msg == 'help' then
		print('--iTargetingFrames--\n/itf from X (string)\n/itf to X (string)\n/itf x X (number)\n/itf y X (number)\n/itf show')
	else
		print('iTF: ' .. msg)
	end
end

function iTF:updateHealth(unitID)
	if iTF.frames[unitID] then
		local hp = UnitHealth(unitID)
		local maxHP = UnitHealthMax(unitID)
		local value = 1
		if hp and maxHP then
			value = hp/maxHP
		end
		iTF.frames[unitID].healthBar:SetValue(value)
	end
end
function iTF:updateAuras(unitID)
	local tempAuraTable = {}
	local function sortAuras()
		--print('sorting')
		local id = 1
		for k,v in spairs(tempAuraTable, function(t,a,b) return t[b].expirationTime > t[a].expirationTime end) do
			if not iTF.frames[unitID].auras[id] then
				iTF:CreateAuraFrame(unitID, id)
			end
			iTF.frames[unitID].auras[id].tex:SetTexture(v.icon)
			iTF.frames[unitID].auras[id].cooldown:SetCooldown(v.start, v.duration)
			iTF.frames[unitID].auras[id]:Show()
			id = id + 1
		end
		for i = id, #iTF.frames[unitID].auras do
			iTF.frames[unitID].auras[i]:Hide()
		end
	end
	for i = 1, 41 do
		local name, _, icon, count, debuffType, duration, expirationTime, _, _, _, spellID = UnitDebuff(unitID, i, 'player')
		if not name then
			sortAuras()
		else
			if not iTF.auraBlacklist[spellID] then
				table.insert(tempAuraTable, {['icon'] = icon, ['duration'] = duration, ['expirationTime'] = expirationTime, ['count'] = count, ['start'] = expirationTime - duration})
			end
		end
	end
end
function iTF:updateCast(unitID,channel)
	local function GetCast()
		if channel then
			return UnitChannelInfo(unitID)
		else
			return UnitCastingInfo(unitID)
		end
	end
	local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = GetCast()
	if name then
		iTF.frames[unitID].text:SetText(string.sub(name, 1, 12))
		if notInterruptible then
			iTF.frames[unitID].text:SetTextColor(iTFConfig.colors.text.immune.r,iTFConfig.colors.text.immune.g,iTFConfig.colors.text.immune.b,iTFConfig.colors.text.immune.a)
		else
			iTF.frames[unitID].text:SetTextColor(iTFConfig.colors.text.cast.r,iTFConfig.colors.text.cast.g,iTFConfig.colors.text.cast.b,iTFConfig.colors.text.cast.a)
		end
		iTF.frames[unitID].healthBar:SetHeight(iTFConfig.frame.height-1-iTFConfig.frame.castBarHeight)
		iTF.frames[unitID].castBar:Show()
		iTF.frames[unitID].castBar:SetMinMaxValues(startTime/1000, endTime/1000)
		iTF.frames[unitID].castBar:SetScript('OnUpdate', function()			
			iTF.frames[unitID].castBar:SetValue(GetTime())
		end)
	else
		if UnitExists(unitID) then
			iTF.frames[unitID].text:SetText(string.sub(UnitName(unitID), 1, 10))
			iTF.frames[unitID].text:SetTextColor(iTFConfig.colors.text.main.r,iTFConfig.colors.text.main.g,iTFConfig.colors.text.main.b,iTFConfig.colors.text.main.a)
		end
		iTF.frames[unitID].castBar:SetScript('OnUpdate', nil)
		iTF.frames[unitID].castBar:SetValue(0)
		iTF.frames[unitID].castBar:Hide()
		iTF.frames[unitID].healthBar:SetHeight(iTFConfig.frame.height)
	end
end
function iTF:CreateAuraFrame(unitID, id)
	--Create frame
	iTF.frames[unitID].auras[id] = CreateFrame('frame',nil,iTF.frames[unitID])
	iTF.frames[unitID].auras[id]:SetSize(14,14)
	--Backdrop
	iTF.frames[unitID].auras[id]:SetBackdrop(iTF.backdrop)
	iTF.frames[unitID].auras[id]:SetBackdropColor(iTFConfig.colors.backdrop.bg.r,iTFConfig.colors.backdrop.bg.g,iTFConfig.colors.backdrop.bg.b,iTFConfig.colors.backdrop.bg.a)
	iTF.frames[unitID].auras[id]:SetBackdropBorderColor(iTFConfig.colors.backdrop.border.r,iTFConfig.colors.backdrop.border.g,iTFConfig.colors.backdrop.border.b,iTFConfig.colors.backdrop.border.a)
	iTF.frames[unitID].auras[id]:SetFrameStrata('HIGH')
	if id == 1 then
		iTF.frames[unitID].auras[id]:SetPoint('BOTTOMLEFT', iTF.frames[unitID], 'BOTTOMLEFT', 1, 1)
	else
		iTF.frames[unitID].auras[id]:SetPoint('LEFT', iTF.frames[unitID].auras[id-1], 'RIGHT', 2, 0)
	end
	--Icon
	iTF.frames[unitID].auras[id].tex = iTF.frames[unitID].auras[id]:CreateTexture()
	iTF.frames[unitID].auras[id].tex:SetAllPoints(iTF.frames[unitID].auras[id])
	--Cooldown
	iTF.frames[unitID].auras[id].cooldown = CreateFrame('Cooldown', nil, iTF.frames[unitID].auras[id], 'CooldownFrameTemplate')
	iTF.frames[unitID].auras[id].cooldown:SetAllPoints(iTF.frames[unitID].auras[id])
	iTF.frames[unitID].auras[id].cooldown:SetReverse(true)
	iTF.frames[unitID].auras[id].cooldown:SetHideCountdownNumbers(false)
end
function iTF:getUFPos(id)
	local r = id%iTFConfig.colS
	local posX = math.floor(id/iTFConfig.colS)*(iTFConfig.frame.width+iTFConfig.frame.spacing)
	local posY = (r-1)*(iTFConfig.frame.height+iTFConfig.frame.spacing)
	
	if r == 0 then
		posX = (math.floor(id/iTFConfig.colS)-1)*(iTFConfig.frame.width+iTFConfig.frame.spacing)
		posY = (iTFConfig.colS-1)*(iTFConfig.frame.height+iTFConfig.frame.spacing)
	end
	return posX, posY
end
function iTF:updatePos()
	iTF.mainFrame:SetPoint(iTFConfig.from, UIParent, iTFConfig.to, iTFConfig.x, iTFConfig.y)
end
function iTF:updateSize()
	local i = 1
	for k in pairs(iTF.frames) do
		local posX, posY = iTF:getUFPos(i)
		iTF.frames[k]:SetPoint('TOPLEFT', iTF.mainFrame, 'TOPLEFT', posX, -posY)
		iTF.frames[k]:SetSize(iTFConfig.frame.width,iTFConfig.frame.height)
		iTF.frames[k].healthBar:SetWidth(iTFConfig.frame.width-2)
		iTF.frames[k].healthBar:SetHeight(iTFConfig.frame.height-2)
		iTF.frames[unitID].castBar:SetWidth(iTFConfig.frame.width-2)
		i = i + 1
	end
end
function iTF:updateRaidIcon(unitID)
	local function checkUnit(unitID)
		local raidIcon = GetRaidTargetIndex(unitID)
		if raidIcon and raidIcon < 9 then
			iTF.frames[unitID].raidIcon:SetTexture('Interface\\TargetingFrame\\UI-RaidTargetingIcon_'..raidIcon)
			iTF.frames[unitID].raidIcon:Show()
		else
			iTF.frames[unitID].raidIcon:Hide()
		end
	end
	if unitID then
		checkUnit(unitID)
	else
		for k in pairs(iTF.frames) do
			if UnitExists(k) then
				checkUnit(k)
			end
		end
	end
end
function iTF:CreateNew(unitID, i)
	if not iTF.frames then
		iTF.frames = {}
	end
	--[[
	local r = i%iTFConfig.colS
	local posX = math.floor(i/iTFConfig.colS)*(iTFConfig.frame.width+iTFConfig.frame.spacing)
	local posY = (r-1)*(iTFConfig.frame.height+iTFConfig.frame.spacing)
	
	if r == 0 then
		posY = (iTFConfig.colS-1)*(iTFConfig.frame.height+iTFConfig.frame.spacing)
		posX = (math.floor(i/iTFConfig.colS)-1)*(iTFConfig.frame.width+iTFConfig.frame.spacing)
	end
	--]]
	local posX, posY = iTF:getUFPos(i)
	iTF.frames[unitID] = CreateFrame('BUTTON', nil, iTF.mainFrame, 'SecureUnitButtonTemplate')
	iTF.frames[unitID]:SetAttribute('unit', unitID)
	iTF.frames[unitID]:SetAttribute('type1', 'target')
	iTF.frames[unitID]:SetSize(iTFConfig.frame.width+2,iTFConfig.frame.height+2)
	iTF.frames[unitID]:SetPoint('TOPLEFT', iTF.mainFrame, 'TOPLEFT', posX, -posY)
	
	--Backdrop
	iTF.frames[unitID]:SetBackdrop(iTF.backdrop)
	iTF.frames[unitID]:SetBackdropColor(iTFConfig.colors.backdrop.bg.r,iTFConfig.colors.backdrop.bg.g,iTFConfig.colors.backdrop.bg.b,iTFConfig.colors.backdrop.bg.a)
	iTF.frames[unitID]:SetBackdropBorderColor(iTFConfig.colors.backdrop.border.r,iTFConfig.colors.backdrop.border.g,iTFConfig.colors.backdrop.border.b,iTFConfig.colors.backdrop.border.a)
	
	--Health Bar
	iTF.frames[unitID].healthBar = CreateFrame('StatusBar', nil,iTF.frames[unitID])
	iTF.frames[unitID].healthBar:SetStatusBarTexture(iTFConfig.statusbar.texture)
	iTF.frames[unitID].healthBar:SetWidth(iTFConfig.frame.width)
	iTF.frames[unitID].healthBar:SetHeight(iTFConfig.frame.height)
	iTF.frames[unitID].healthBar:SetPoint('BOTTOMLEFT', iTF.frames[unitID], 'BOTTOMLEFT', 1,1)
	iTF.frames[unitID].healthBar:SetStatusBarColor(iTFConfig.colors.statusbar.main.r,iTFConfig.colors.statusbar.main.g,iTFConfig.colors.statusbar.main.b,iTFConfig.colors.statusbar.main.a)
	iTF.frames[unitID].healthBar:SetMinMaxValues(0,1)
	
	--Name text
	iTF.frames[unitID].text = iTF.frames[unitID].healthBar:CreateFontString()
	iTF.frames[unitID].text:SetFont(iTFConfig.font.name, iTFConfig.font.size, 'OUTLINE')
	iTF.frames[unitID].text:SetPoint('TOPLEFT', iTF.frames[unitID], 'TOPLEFT', 2,-2)
	iTF.frames[unitID].text:SetText('test')
	iTF.frames[unitID].text:SetTextColor(iTFConfig.colors.text.main.r,iTFConfig.colors.text.main.g,iTFConfig.colors.text.main.b,iTFConfig.colors.text.main.a)
	
	--Cast Bar
	iTF.frames[unitID].castBar = CreateFrame('StatusBar', nil,iTF.frames[unitID])
	iTF.frames[unitID].castBar:SetStatusBarTexture(iTFConfig.statusbar.texture)
	iTF.frames[unitID].castBar:SetWidth(iTFConfig.frame.width)
	iTF.frames[unitID].castBar:SetHeight(iTFConfig.frame.castBarHeight)
	iTF.frames[unitID].castBar:SetPoint('TOPLEFT', iTF.frames[unitID], 'TOPLEFT', 1,-1)
	iTF.frames[unitID].castBar:SetStatusBarColor(iTFConfig.colors.statusbar.cast.r,iTFConfig.colors.statusbar.cast.g,iTFConfig.colors.statusbar.cast.b,iTFConfig.colors.statusbar.cast.a)
	iTF.frames[unitID].castBar:SetMinMaxValues(0,1)
	iTF.frames[unitID].castBar:SetValue(0)
	iTF.frames[unitID].castBar:Hide()
	
	--Glow
	iTF.frames[unitID].topFrame = CreateFrame('frame', nil, iTF.frames[unitID]) -- anchor frame
	iTF.frames[unitID].topFrame:SetAllPoints(iTF.frames[unitID])
	iTF.frames[unitID].topFrame:SetFrameStrata('high')
	--Top
	iTF.frames[unitID].topFrame.glowTop = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowTop:SetPoint('TOP', iTF.frames[unitID].topFrame, 'TOP', 0,-1)
	iTF.frames[unitID].topFrame.glowTop:SetSize(iTFConfig.frame.width, 8)
	iTF.frames[unitID].topFrame.glowTop:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\hGlow')
	iTF.frames[unitID].topFrame.glowTop:SetTexCoord(0, 60/64, 0, 1)
	iTF.frames[unitID].topFrame.glowTop:SetVertexColor(1,1,1,0.75)
	iTF.frames[unitID].topFrame.glowTop:Hide()
	--Bottom
	iTF.frames[unitID].topFrame.glowBottom = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowBottom:SetPoint('BOTTOM', iTF.frames[unitID].topFrame, 'BOTTOM', 0,1)
	iTF.frames[unitID].topFrame.glowBottom:SetSize(iTFConfig.frame.width, 8)
	iTF.frames[unitID].topFrame.glowBottom:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\hGlow')
	iTF.frames[unitID].topFrame.glowBottom:SetTexCoord(0, 60/64, 1, 0)
	iTF.frames[unitID].topFrame.glowBottom:SetVertexColor(1,1,1,0.75)
	iTF.frames[unitID].topFrame.glowBottom:Hide()
	--Left
	iTF.frames[unitID].topFrame.glowLeft = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowLeft:SetPoint('LEFT', iTF.frames[unitID].topFrame, 'LEFT', 1,0)
	iTF.frames[unitID].topFrame.glowLeft:SetSize(8, iTFConfig.frame.height)
	iTF.frames[unitID].topFrame.glowLeft:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\hGlow')
	iTF.frames[unitID].topFrame.glowLeft:SetTexCoord(60/64,0,0,0,60/64,1,0,1)
	iTF.frames[unitID].topFrame.glowLeft:SetVertexColor(1,1,1,0.75)
	iTF.frames[unitID].topFrame.glowLeft:Hide()
	--Right
	iTF.frames[unitID].topFrame.glowRight = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowRight:SetPoint('RIGHT', iTF.frames[unitID].topFrame, 'RIGHT', -1,0)
	iTF.frames[unitID].topFrame.glowRight:SetSize(8, iTFConfig.frame.height)
	iTF.frames[unitID].topFrame.glowRight:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\hGlow')
	iTF.frames[unitID].topFrame.glowRight:SetTexCoord(0,1,60/64,1,0,0,60/64,0)
	iTF.frames[unitID].topFrame.glowRight:SetVertexColor(1,1,1,0.75)
	iTF.frames[unitID].topFrame.glowRight:Hide()
	--[[
	--Top left
	iTF.frames[unitID].topFrame.glowTopLeft = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowTopLeft:SetPoint('TOPLEFT', iTF.frames[unitID].topFrame, 'TOPLEFT', 1,-1)
	iTF.frames[unitID].topFrame.glowTopLeft:SetSize(24, 24)
	iTF.frames[unitID].topFrame.glowTopLeft:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\cGlow')
	iTF.frames[unitID].topFrame.glowTopLeft:SetTexCoord(1, 0, 0, 1)
	iTF.frames[unitID].topFrame.glowTopLeft:SetVertexColor(0,1,1,1)
	--Top right
	iTF.frames[unitID].topFrame.glowTopRight = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowTopRight:SetPoint('TOPRIGHT', iTF.frames[unitID].topFrame, 'TOPRIGHT', -1,-1)
	iTF.frames[unitID].topFrame.glowTopRight:SetSize(24, 24)
	iTF.frames[unitID].topFrame.glowTopRight:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\cGlow')
	iTF.frames[unitID].topFrame.glowTopRight:SetTexCoord(0, 1, 0, 1)
	iTF.frames[unitID].topFrame.glowTopRight:SetVertexColor(1,0,1,1)
	--Bottom left
	iTF.frames[unitID].topFrame.glowBottomLeft = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowBottomLeft:SetPoint('BOTTOMLEFT', iTF.frames[unitID].topFrame, 'BOTTOMLEFT', 1, 1)
	iTF.frames[unitID].topFrame.glowBottomLeft:SetSize(8, 8)
	iTF.frames[unitID].topFrame.glowBottomLeft:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\cGlow')
	iTF.frames[unitID].topFrame.glowBottomLeft:SetTexCoord(1, 0, 1, 0)
	iTF.frames[unitID].topFrame.glowBottomLeft:SetVertexColor(1,0,1,1)
	--Bottom right
	iTF.frames[unitID].topFrame.glowBottomRight = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].topFrame.glowBottomRight:SetPoint('BOTTOMRIGHT', iTF.frames[unitID].topFrame, 'BOTTOMRIGHT', -1, 1)
	iTF.frames[unitID].topFrame.glowBottomRight:SetSize(8, 8)
	iTF.frames[unitID].topFrame.glowBottomRight:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\cGlow')
	iTF.frames[unitID].topFrame.glowBottomRight:SetTexCoord(0, 1, 1, 0)
	iTF.frames[unitID].topFrame.glowBottomRight:SetVertexColor(1,1,0,1)
	--]]
	--Raid Icon
	iTF.frames[unitID].raidIcon = iTF.frames[unitID].topFrame:CreateTexture()
	iTF.frames[unitID].raidIcon:SetSize(iTFConfig.raidIcon.size, iTFConfig.raidIcon.size)
	iTF.frames[unitID].raidIcon:SetPoint('CENTER', iTF.frames[unitID].topFrame, 'CENTER', 0,0)
	iTF.frames[unitID].raidIcon:Hide()
	--iTF.frames[unitID].raidIcon:SetTexture('Interface\\TargetingFrame\\UI-RaidTargetingIcon_8')
	--Auras
	iTF.frames[unitID].auras = {}
	
	--Hide & Scripts
	iTF.frames[unitID]:Hide()
	iTF.frames[unitID]:SetScript('OnShow',function()
		iTF.frames[unitID].text:SetText(string.sub(UnitName(unitID), 1, 10))
		iTF:updateHealth(unitID)
		iTF:updateAuras(unitID)
		iTF:updateCast(unitID)
		iTF:updateRaidIcon(unitID)
	end)
	RegisterUnitWatch(iTF.frames[unitID])
end
function iTF.glowColor(unitID, type)

end
local onUpdateTotal = 0
function iTF:OnUpdate(elapsed)
	onUpdateTotal = onUpdateTotal + elapsed
	if onUpdateTotal >= 0.2 then
		for k in pairs(iTF.frames) do
			if UnitExists(k) and not UnitIsUnit(k, 'player') then 
				--if IsSpellInRange('Dark Command', k) == 1 then
				if iTF.rangeSpells[iTF.specID] then
					if type(iTF.rangeSpells[iTF.specID]) == 'table' then
						if IsSpellInRange(iTF.rangeSpells[iTF.specID].min, k) == 1 or IsSpellInRange(iTF.rangeSpells[iTF.specID].min, k) == 1 then
							iTF.frames[k]:SetAlpha(1)
						else
							iTF.frames[k]:SetAlpha(0.3)
						end
					elseif IsSpellInRange(iTF.rangeSpells[iTF.specID], k) == 1 then
						iTF.frames[k]:SetAlpha(1)
					else
						iTF.frames[k]:SetAlpha(0.3)
					end
				else
					iTF.frames[k]:SetAlpha(1)
				end
				if not iTF.encounterInProgress and not UnitAffectingCombat(k) then
					iTF.frames[k].topFrame.glowLeft:SetVertexColor(0,1,1,0.8)
					iTF.frames[k].topFrame.glowLeft:Show()
				else
					iTF.frames[k].topFrame.glowLeft:Hide()
				end
			end
		end
		iTEST = iTF
		onUpdateTotal = 0
	end
end
function iTF:CreateMainFrame()
	iTF.mainFrame = CreateFrame('frame', nil, UIParent)
	local width = (iTFConfig.frame.width+iTFConfig.frame.spacing)*math.ceil(30/iTFConfig.colS)
	local height = (iTFConfig.frame.height+iTFConfig.frame.spacing)*iTFConfig.colS
	iTF.mainFrame:SetSize(width, height)
	--iTF.mainFrame:SetWidth(60)
	--iTF.mainFrame:SetHeight(1)
	iTF.mainFrame:SetPoint(iTFConfig.from, UIParent, iTFConfig.to, iTFConfig.x, iTFConfig.y)
	iTF.mainFrame.tex = iTF.mainFrame:CreateTexture()
	iTF.mainFrame.tex:SetAllPoints(iTF.mainFrame)
	--local tex = 'Interface\\AddOns\\iTargetingFrames\\media\\LtoR'
	--iTF.mainFrame.tex:SetTexture('Interface\\AddOns\\iTargetingFrames\\media\\LtoR')
	--iTF.mainFrame.tex:SetVertexColor(1,0,0,1)
	--local function animationOnUpdate(self, elapsed)
	--	AnimateTexCoords(self.tex, 128, 128, 60, 1, 120, elapsed, 0.005)
	--end
	--iTF.mainFrame:SetScript('OnUpdate', animationOnUpdate)
	iTF.mainFrame.tex:SetColorTexture(0.5,0.5,0.5,0.5)
	iTF.mainFrame.tex:Hide()
	for i = 1, 30 do --Create 30 frames
		iTF:CreateNew('nameplate' .. i,i)
	end
	iTF.mainFrame:SetScript('OnUpdate', iTF.OnUpdate)
end
function addon:ADDON_LOADED(addonName)
	if addonName == 'iTargetingFrames' then
		iTFConfig = iTFConfig or {}
		iTF:LoadDefaults()
		addon:UnregisterEvent('ADDON_LOADED')
		iTF:CreateMainFrame()
		addon:RegisterEvent('UNIT_HEALTH')
		addon:RegisterEvent('UNIT_HEALTH_FREQUENT')
		addon:RegisterEvent('UNIT_SPELLCAST_START')
		addon:RegisterEvent('UNIT_SPELLCAST_STOP')
		addon:RegisterEvent('UNIT_SPELLCAST_INTERRUPTED')
		addon:RegisterEvent('UNIT_SPELLCAST_CHANNEL_START')
		addon:RegisterEvent('UNIT_SPELLCAST_CHANNEL_STOP')
		addon:RegisterEvent('UNIT_AURA')
		addon:RegisterEvent('ENCOUNTER_START')
		addon:RegisterEvent('ENCOUNTER_STOP')
		addon:RegisterEvent('RAID_TARGET_UPDATE')
		addon:RegisterEvent('PLAYER_LOGIN')
		addon:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED')
	end
end
function addon:UNIT_HEALTH_FREQUENT(unitID)
	if iTF.frames[unitID] then
		iTF:updateHealth(unitID)
	end
end
function addon:UNIT_HEALTH(unitID)
	if iTF.frames[unitID] then
		iTF:updateHealth(unitID)
	end
end
function addon:UNIT_AURA(unitID)
	if iTF.frames[unitID] then
		iTF:updateAuras(unitID)
	end
end
function addon:UNIT_SPELLCAST_START(unitID)
	if iTF.frames[unitID] then
		iTF:updateCast(unitID)
	end
end
function addon:UNIT_SPELLCAST_STOP(unitID)
	if iTF.frames[unitID] then
		iTF:updateCast(unitID)
	end
end
function addon:UNIT_SPELLCAST_INTERRUPTED(unitID)
	if iTF.frames[unitID] then
		iTF:updateCast(unitID)
	end
end
function addon:UNIT_SPELLCAST_CHANNEL_START(unitID)
	if iTF.frames[unitID] then
		iTF:updateCast(unitID, true)
	end
end
function addon:UNIT_SPELLCAST_CHANNEL_STOP(unitID)
	if iTF.frames[unitID] then
		iTF:updateCast(unitID, true)
	end
end
function addon:ENCOUNTER_START()
	iTF.encounterInProgress = true
end
function addon:ENCOUNTER_STOP()
	iTF.encounterInProgress = false
end
function addon:RAID_TARGET_UPDATE()
	iTF:updateRaidIcon()
end
function addon:PLAYER_LOGIN()
	iTF.specID = GetSpecializationInfo(GetSpecialization())
end
function addon:PLAYER_SPECIALIZATION_CHANGED()
	iTF.specID = GetSpecializationInfo(GetSpecialization())
end
function iP()
	local spellID = select(3,GameTooltip:GetSpell())
	local specID = GetSpecializationInfo(GetSpecialization())
	print(specID, spellID)
end
SLASH_ITF1 = "/itf"
SLASH_ITF2 = '/itargetingframes'
SlashCmdList["ITF"] = function(realMsg)
	--[[
	local msg = realMsg
	if msg then 
		msg = string.lower(msg) 
	else
		iTF:print(string.format('Command "%s" not found. Try /itf help', ' '))
		return
	end
	local function TrimWS(str)
		return str:gsub('^%s*(.-)%s*$', '%1')
	end
	if string.match(msg, 'to') then
		local value = string.gsub(msg, '^to ', '')
		iTFConfig.to = TrimWS(value)
		iTF:updatePos()
	elseif string.match(msg, 'from') then
		local value = string.gsub(msg, '^to ', '')
		iTFConfig.from = TrimWS(value)
		iTF:updatePos()
	elseif string.match(msg, 'x') then
		local value = string.gsub(msg, '^x ', '')
		iTFConfig.x = tonumber(value)
		iTF:updatePos()
	elseif string.match(msg, 'y') then
		local value = string.gsub(msg, '^y ', '')
		iTFConfig.y = tonumber(value)
		iTF:updatePos()
	elseif msg == 'help' then
		iTF:print('help')
	elseif msg == 'show' then
		if iTF.show then
			iTF.mainFrame.tex:Hide()
			iTF.show = nil
		else
			iTF.mainFrame.tex:Show()
			iTF.show = true
		end
	else
	
		iTF:print(string.format('Command "%s" not found. Try /itf help', msg))
	end
	--]]
	iTF:print('Options are currently disabled, modify wtf file if you know what you are doing.')
end